#!/bin/bash
basename="$1"
units="$2"
native="$3"
cluster_threshold=${ML_CLUSTER_THRESHOLD:-10}

export ML_BIN_DIR=${ML_BIN_DIR:-$(dirname "$0")}

if ! [ -f "${ML_BIN_DIR}/multilzerd" ]; then
	2>&1 echo "[error] Could not find the Multi-LZerD directory. Set the ML_BIN_DIR environment variable to the directory where you put the Multi-LZerD executables."
	exit 1
fi

cp -n "${ML_BIN_DIR}/uniCHARMM" .
cp -n "${ML_BIN_DIR}/charmm_atom_types.in" .
cp -n "${ML_BIN_DIR}/charmm_params.in" .

echo 'Adding hydrogens...'
"${ML_BIN_DIR}/addhydrogens.pl" $basename $units

echo 'Creating pairwise predictions with LZerD'
date
"${ML_BIN_DIR}/create_lzerd_decoys.pl" $basename $units execute

echo 'Filtering pairwise predictions'
date
"${ML_BIN_DIR}/multilzerd_sort_decoys.pl" ./ $units 54000

echo 'Clustering pairwise predictions'
date
"${ML_BIN_DIR}/multilzerd_pairwise_cluster.pl" ./ $basename $units cluster $cluster_threshold allrmsd nosizes execute

rm *.out
rename s/cluster/out/ *.cluster

ML_NUM_GENERATIONS=${ML_NUM_GENERATIONS:-3000}

echo 'Multi-LZerD...'
date
"${ML_BIN_DIR}/multilzerd" --pdbid $basename --chains $units -o $basename --generations ${ML_NUM_GENERATIONS} --population 200 --clashes 200 --cluster $cluster_threshold --weights all --detail

finalgaf=$(printf "%05d-${basename}.ga.out" ${ML_NUM_GENERATIONS})

echo 'Creating PDB files for final decoys'
date
"${ML_BIN_DIR}/multilzerd_create_pdb" ${finalgaf} ./ 1 200 decoy

# final generation models
cp "${finalgaf}" "final.ga.out"

# lowest-rmsd model
if [[ "${native}" != "" ]]; then
	echo "Calculating complex RMSDs"
	date

	if ! [ -f "${native}" ]; then
		2>&1 echo "[error] native structure file '${native}' does not exist"
		exit 1
	elif ! [ -v DATADIR ]; then
		2>&1 echo "[error] the DATADIR environment variable must be set to calculate RMSDs with ProFit"
		exit 1
	fi

	for f in decoy-*.pdb; do
		rmsd=$(echo -e "REFE ${native}\n MOBI ${f}\n ATOMS CA,C,N,O\n IGNOREMISSING\n ALIGN\n FIT\n" | profit | grep "RMS:" | colrm 1 8)
		echo "${f} ${rmsd}"
	done > ml.rmsd

	lidx=$(sort -k2 -n ml.rmsd | head -n1 | awk '{print $1}' | egrep -o '[1-9][0-9]*')
	lgaline="$(cat ${finalgaf} | tail -n +9 | head -n ${lidx} | tail -n 1)"
	head -n 8 ${finalgaf} > lowest.ga.out
	echo "${lgaline}" >> lowest.ga.out
fi

# consensus models
ML_CONSENSUS_SIZE=${ML_CONSENSUS_SIZE:-1000}
upper=${ML_NUM_GENERATIONS}
lower=$(( ${ML_NUM_GENERATIONS} - ${ML_CONSENSUS_SIZE} ))
if [ ${lower} -lt 1 ]; then
	lower=1
fi

head -n 8 ${finalgaf} > consensus.ga.out
for i in $(seq $lower $upper); do
	tail -n +9 $(printf "%05d-${basename}.ga.out" $i)
done >> consensus.ga.out
